/* $LastChangedRevision: 10 $
 * $LastChangedDate:: 2016-01-13 08:47:25#$
 *
 * Copyright (c) 2013, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */

#include "Target.h"
#include "GlobalDefines.h"
#include "Hardware.h"

/* private prototypes */

/* private macros */

/* configuration bits */
#if defined(__EP100__)
    // FICD
    #pragma config ICS = PGD2               // ICD Communication Channel Select bits (Communicate on PGEC2 and PGED2)
    #pragma config JTAGEN = OFF             // JTAG Enable bit (JTAG is disabled)

    // FPOR
    #pragma config ALTI2C1 = OFF            // Alternate I2C1 pins (I2C1 mapped to SDA1/SCL1 pins)
    #pragma config ALTI2C2 = OFF            // Alternate I2C2 pins (I2C2 mapped to SDA2/SCL2 pins)
    #pragma config WDTWIN = WIN25           // Watchdog Window Select bits (WDT Window is 25% of WDT period)

    // FWDT
    #pragma config WDTPOST = PS16           // Watchdog Timer Postscaler bits (1:16)
    #pragma config WDTPRE = PR32            // Watchdog Timer Prescaler bit (1:32) -> pre- and postscale result in 16ms watchdog timeout (1/32000*16*32)
    #pragma config PLLKEN = ON              // PLL Lock Enable bit (Clock switch to PLL source will wait until the PLL lock signal is valid.)
    #pragma config WINDIS = OFF             // Watchdog Timer Window Enable bit (Watchdog Timer in Non-Window mode)
    #pragma config FWDTEN = OFF             // Watchdog Timer Enable bit (Watchdog timer enabled/disabled by user software)

    // FOSC
    //#pragma config POSCMD = HS              // Primary Oscillator Mode Select bits (HS Crystal Oscillator Mode)
    #pragma config OSCIOFNC = OFF           // OSC2 Pin Function bit (OSC2 is clock output)
    #pragma config IOL1WAY = OFF            // Peripheral pin select configuration (Allow multiple reconfigurations)
    #pragma config FCKSM = CSECME           // Clock Switching Mode bits (Both Clock switching and Fail-safe Clock Monitor are enabled)

    // FOSCSEL
    //#pragma config FNOSC = PRIPLL           // Oscillator Source Selection (Primary Oscillator with PLL module (XT + PLL, HS + PLL, EC + PLL))
    #pragma config IESO = OFF               // Two-speed Oscillator Start-up Enable bit (Start up with user-selected oscillator source)
    #pragma config PWMLOCK = OFF            // PWM registers my be written without key sequence

    // FGS
    #pragma config GWRP = OFF               // General Segment Write-Protect bit (General Segment may be written)
    #pragma config GCP = OFF                // General Segment Code-Protect bit (General Segment Code protect is Disabled)

    #pragma config FNOSC = FRC              // Oscillator Source Selection (Internal Fast RC (FRC))
    #pragma config POSCMD = NONE            // No primary oscillator
    #define OSC_SOURCE 0x01                 // Fast RC oscillator with PLL
#elif defined(__EP200__)
    /* DSPIC33EV128GM106 Configuration Bit Settings */
    // FSEC
    #pragma config BWRP = OFF               // Boot Segment Write-Protect Bit (Boot Segment may be written)
    #pragma config BSS = DISABLED           // Boot Segment Code-Protect Level bits (No Protection (other than BWRP))
    #pragma config BSS2 = OFF               // Boot Segment Control Bit (No Boot Segment)
    #pragma config GWRP = OFF               // General Segment Write-Protect Bit (General Segment may be written)
    #pragma config GSS = DISABLED           // General Segment Code-Protect Level bits (No Protection (other than GWRP))
    #pragma config CWRP = OFF               // Configuration Segment Write-Protect Bit (Configuration Segment may be written)
    #pragma config CSS = DISABLED           // Configuration Segment Code-Protect Level bits (No Protection (other than CWRP))
    #pragma config AIVTDIS = DISABLE        // Alternate Interrupt Vector Table Disable Bit  (Disable Alternate Vector Table)

    // FBSLIM
    #pragma config BSLIM = 0x1FFF           // Boot Segment Code Flash Page Address Limit Bits (Boot Segment Flash Page Address Limit (0-0x1FFF))

    // FOSCSEL
    #pragma config FNOSC = PRIPLL           // Initial oscillator Source Selection Bits (Primary Oscillator with PLL module (XT + PLL, HS + PLL, EC + PLL))
    #pragma config IESO = OFF               // Two Speed Oscillator Start-Up Bit (Start up device with user selected oscillator source)

    // FOSC
    #pragma config POSCMD = HS              // Primary Oscillator Mode Select Bits (HS Crystal Oscillator mode)
    #pragma config OSCIOFNC = OFF           // OSC2 Pin I/O Function Enable Bit (OSC2 is clock output)
    #pragma config IOL1WAY = OFF            // Peripheral pin select configuration (Allow multiple reconfigurations)
    #pragma config FCKSM = CSECME           // Clock Switching Mode bits (Both Clock switching and Fail-safe Clock Monitor are enabled)
    #pragma config PLLKEN = ON              // PLL Lock Enable Bit (Clock switch to PLL source will wait until the PLL lock signal is valid)

    // FWDT
    #pragma config WDTPOST = PS16           // Watchdog Timer Postscaler Bits (1:16)
    #pragma config WDTPRE = PR32            // Watchdog Timer Prescaler Bit (1:32) -> pre- and postscale result in 16ms watchdog timeout (1/32000*16*32)
    #pragma config FWDTEN = OFF             // Watchdog Timer Enable bit (Watchdog timer enabled/disabled by user software)
    #pragma config WINDIS = OFF             // Watchdog Timer Window Enable Bit (Watchdog timer in Non-Window Mode)
    #pragma config WDTWIN = WIN25           // Watchdog Window Select Bits (WDT Window is 25% of WDT period)

    // FPOR
    #pragma config BOREN0 = ON              // Brown Out Reset Detection Bit (BOR is Enabled)

    // FICD
    #pragma config ICS = PGD2               // ICD Communication Channel Select Bits (Communicate on PGEC2 and PGED2)

    // FDMTINTVL
    #pragma config DMTIVTL = 0xFFFF         // Lower 16 Bits of 32 Bit DMT Window Interval (Lower 16 bits of 32 bit DMT window interval (0-0xFFFF))

    // FDMTINTVH
    #pragma config DMTIVTH = 0xFFFF         // Upper 16 Bits of 32 Bit DMT Window Interval (Upper 16 bits of 32 bit DMT window interval (0-0xFFFF))

    // FDMTCNTL
    #pragma config DMTCNTL = 0xFFFF         // Lower 16 Bits of 32 Bit DMT Instruction Count Time-Out Value (Lower 16 bits of 32 bit DMT instruction count time-out value (0-0xFFFF))

    // FDMTCNTH
    #pragma config DMTCNTH = 0xFFFF         // Upper 16 Bits of 32 Bit DMT Instruction Count Time-Out Value (Upper 16 bits of 32 bit DMT instruction count time-out value (0-0xFFFF))

    // FDMT
    #pragma config DMTEN = DISABLE          // Dead Man Timer Enable Bit (Dead Man Timer is Disabled and can be enabled by software)

    // FDEVOPT
    #pragma config PWMLOCK = OFF            // PWM Lock Enable Bit (PWM registers may be written without key sequence)
    #pragma config ALTI2C1 = OFF            // Alternate I2C1 Pins Selection Bit (I2C1 mapped to SDA1/SCL1 pins)

    // FALTREG
    #pragma config CTXT1 = NONE             // Interrupt Priority Level (IPL) Selection Bits For Alternate Working Register Set 1 (Not Assigned)
    #pragma config CTXT2 = NONE             // Interrupt Priority Level (IPL) Selection Bits For Alternate Working Register Set 2 (Not Assigned)

    #define OSC_SOURCE 0x03                 // Primary oscillator with PLL
#else
    #error MOTOR TYPE NOT DEFINED
#endif
    
    

/** 
 * @brief Hardware initialization.
 */
void initHardware(void)
{
    /*------------------------------------------------------------------------*/
    /**
     *  - Configuration of oscillator
     *    - Internal oscillator (fast RC oscillator with PLL) for EP100
     *    - External oscillator (high-speed oscillator with PLL) for EP200
     *    - fCY = 40MHz or 70MHz (selectable via pre-processor macro)
     *    .
     */
    /*------------------------------------------------------------------------*/
    #if defined(__70MHZ__)
        /* Tune oscillator */
        OSCTUNbits.TUN = 23;        /* 7.37MHz + 23*0.00375*7.37MHz = 8MHz */
        
        /* Configure PLL prescaler, PLL postscaler, PLL divisor */
        CLKDIVbits.PLLPRE = 0;  /* N1 = 0+2 = 2 */
        PLLFBDbits.PLLDIV = 74; /* M = 74+2 = 76 */
        CLKDIVbits.PLLPOST = 0; /* N2 = 2*(0+1) = 2 */

        /* fIn = 7.37MHz
         * fPLLI = fIn / N1 = 3.685MHz => OK
         * fVCO = fIn * (M / N1) = 280MHz => OK
         * fPLLO = fIn * (M/(N1*N2)) = 140MHz => OK
         * fOSC = fPLLO = 140MHz
         * fCY = fOSC/2 = 70MHz
         */
    #elif defined(__40MHZ__)
        #if defined(__EP100__)
            /* Configure PLL prescaler, PLL postscaler, PLL divisor */
            CLKDIVbits.PLLPRE = 1;  /* N1 = 1+2 = 3 */
            PLLFBDbits.PLLDIV = 63; /* M = 63+2 = 65 */
            CLKDIVbits.PLLPOST = 0; /* N2 = 2*(0+1) = 2 */

            /* fIn = 7.37MHz
             * fPLLI = fIn / N1 = 2.4567MHz => OK
             * fVCO = fIn * (M / N1) = 160MHz (159.6833MHz) => OK
             * fPLLO = fIn * (M/(N1*N2)) = 80MHz (798417MHz) => OK
             * fOSC = fPLLO = 80MHz
             * fCY = fOSC/2 = 40MHz
             */
        #elif defined(__EP200__)
            /* Configure PLL prescaler, PLL postscaler, PLL divisor */
            CLKDIVbits.PLLPRE = 2;  /* N1 = 2+2 = 4 */
            PLLFBDbits.PLLDIV = 38; /* M = 38+2 = 40 */
            CLKDIVbits.PLLPOST = 0; /* N2 = 2*(0+1) = 2 */

            /* fIn = 16MHz
             * fPLLI = fIn / N1 = 4MHz => OK
             * fVCO = fIn * (M / N1) = 160MHz => OK
             * fPLLO = fIn * (M/(N1*N2)) = 80MHz => OK
             * fOSC = fPLLO = 80MHz
             * fCY = fOSC/2 = 40MHz
             */
        #else
            #error MOTOR TYPE NOT DEFINED
        #endif
    #else
        #error CLOCK NOT DEFINED
    #endif

   
    /* oscillator source selection */
    __builtin_write_OSCCONH(OSC_SOURCE);
    /* enable switch */
    __builtin_write_OSCCONL(OSCCON | 0x01);

    /* wait for new oscillator to become clock source */
    while(OSCCONbits.COSC != OSC_SOURCE);
    /* wait for PLL to lock */
    while(OSCCONbits.LOCK != 1);

    /* enable watchdog */
    RCONbits.SWDTEN = 1;

    
    /*------------------------------------------------------------------------*/
    /** - Configuration of IO ports */
    /*------------------------------------------------------------------------*/
    /* UART 1 */
    TRISCbits.TRISC5 = 1;   /* UART1 Rx: RC5 as input */
    TRISAbits.TRISA4 = 0;   /* UART1 Tx: RA4 as ouput */
    
    /* Lock output */
    LATCbits.LATC7 = 1;     /* default: set lock output */
    TRISCbits.TRISC7 = 0;   /* RC7 as output */
    
    /* Digital_In */
    TRISCbits.TRISC8 = 1;   /* RC8 as input */
    
    /* ILIMIT_OUT */
    #if defined (__EP100__)
        TRISAbits.TRISA9 = 1;   /* RA9 as input */
    #elif defined(__EP200__)
        TRISCbits.TRISC13 = 1;  /* RC13 as input */
    #endif
    
    /* ZC_OUT */
    TRISAbits.TRISA8 = 1;   /* RA8 as input */
    
    /* Chip (driver) enable */
    LATAbits.LATA7 = 0;     /* default: disable driver */
    TRISAbits.TRISA7 = 0;   /* RA7 as output */
    
//    /* FLT32 (tripzone) */
//    CNPDBbits.CNPDB4 = 1;   /* enable pull down of FLT32 pin -> reset fault */
//    TRISBbits.TRISB4 = 1;   /* RB4 as input */
    
        /* FLT32 (tripzone) */
    CNPUBbits.CNPUB4 = 1;   /* enable pull up of FLT32 pin -> reset fault */
    TRISBbits.TRISB4 = 1;   /* RB4 as input */
    
#if defined(__EP200__)
    /* Standby of CAN-Transceiver */
    LATDbits.LATD8 = 1;     /* default: put transceiver in stand-by */
    TRISDbits.TRISD8 = 0;   /* RD8 as output */
#endif
    
    /* Analog inputs */
    ANSELA = ANSELB = ANSELC = 0x0000;   /* disable all analog inputs */
    ANSELAbits.ANSA0 = 1;
    ANSELAbits.ANSA1 = 1;
    ANSELBbits.ANSB0 = 1;
    ANSELBbits.ANSB1 = 1;
    ANSELBbits.ANSB2 = 1;
    ANSELBbits.ANSB3 = 1;
    ANSELCbits.ANSC0 = 1;
    ANSELCbits.ANSC1 = 1;
    ANSELCbits.ANSC2 = 1;
    
    /* PWM outputs */
    TRISBbits.TRISB14 = 0;
    TRISBbits.TRISB15 = 0;
    TRISBbits.TRISB12 = 0;
    TRISBbits.TRISB13 = 0;
    TRISBbits.TRISB10 = 0;
    TRISBbits.TRISB11 = 0;
    LATBbits.LATB14 = 0;
    LATBbits.LATB15 = 0;
    LATBbits.LATB12 = 0;
    LATBbits.LATB13 = 0;
    LATBbits.LATB10 = 0;
    LATBbits.LATB11 = 0;
    
    
    /*------------------------------------------------------------------------*/
    /** - Configuration of internal OP-Amp */
    /*------------------------------------------------------------------------*/
#if defined(__EP100__)
    CM1CONbits.CON = 0;     /* disable op amp */
    CM1CONbits.OPMODE = 1;  /* circuit acts as an op amp */
    CM1CONbits.CON = 1;     /* enable op amp */
    
    CM3CONbits.CON = 0;     /* disable op amp */
    CM3CONbits.OPMODE = 1;  /* circuit acts as an op amp */
    CM3CONbits.CON = 1;     /* enable op amp */
#elif defined(__EP200__)
    CM1CONbits.CON = 0;     /* disable op amp */
    CM1CONbits.OPAEN = 1;   /* circuit acts as an op amp */
    CM1CONbits.CON = 1;     /* enable op amp */
    
    CM3CONbits.CON = 0;     /* disable op amp */
    CM3CONbits.OPAEN = 1;   /* circuit acts as an op amp */
    CM3CONbits.CON = 1;     /* enable op amp */
#else
    #error MOTOR TYPE NOT DEFINED
#endif

    
      
    /*------------------------------------------------------------------------*/
    /** - Configuration of ADC
     *    - 10 bit mode
     *    - ~1100kS/s
     *    - PWM event starts ADC conversion
     *    - measured time between start and end of conversion: 4.6us
     *    - Channel scanning:
     *      - Phase current 2: AN6 (EP100) / AN3 (EP200)
     *      - Phase current 3: AN3 (EP100) / AN6 (EP200)
     *      - Angle sensor sine:   AN0
     *      - Angle sensor cosine: AN1
     *      - UDC-Link: AN2 (EP100) / AN18 (EP200)
     *      .
     *    .
     */
    /*------------------------------------------------------------------------*/
    AD1CON1bits.ADON = 0;        /* disable ADC */
    
    AD1CON1bits.AD12B = 0;      /* 10bit mode */
    AD1CON1bits.SIMSAM = 1;     /* simultaneously sampling */
    AD1CON1bits.ASAM = 1;       /* automatic sampling (sampling begins after last conversion) */
    AD1CON1bits.SSRCG = 1;      /* sample trigger source group bit */
    AD1CON1bits.SSRC = 0b000;   /* PWM generator 1 primary trigger ends sampling and starts conversion (auto-convert) */  
    AD1CON1bits.FORM = 0b10;    /* fractional output format (=left aligned) */
    AD1CON2bits.VCFG = 0b000;   /* AVdd and AVss as voltage reference */
    AD1CON2bits.CHPS = 0b10;    /* sample channels CH0, CH1, CH2, CH3 */
    AD1CON2bits.SMPI = 1;       /* generate interrupt after completion of every 2nd sample/conversion operation */ //TODO: make distinction between EP100 and EP200!
    AD1CON2bits.CSCNA = 1;      /* input scanning for channel 0 */
    
    AD1CON3bits.ADRC = 0;       /* use system clock for ADC */
    #if defined(__70MHZ__)
        /* Tad = 5 * Tcy = 5 / 70 MHz =~ 71.4 ns */
        /* Txx = Tad * 3 =~ 214.3 ns */
        AD1CON3 = 0x0304;
    #elif defined(__40MHZ__)
        AD1CON3bits.ADCS = 2;   /* Tad = Tcy * (2+1) = 3 / 40 MHz = 75 ns -> Tconv = 12*Tad = 900ns ~1100kS/s */
        AD1CON3bits.SAMC = 3;   /* Tsample = 3 * Tad = 225ns */
    #else
        #error PROCESSOR CLOCK NOT DEFINED
    #endif
    
    AD1CON4bits.ADDMAEN = 0;    /* disable DMA */

    AD1CHS0bits.CH0SA = 2;      /* CH0+: AN2 */
    AD1CHS0bits.CH0NA = 0;      /* CH0-: Vrefl */
    AD1CSSLbits.CSS1 = 1;       /* AN1 */
#if defined(__EP100__)
    AD1CSSLbits.CSS2 = 1;       /* AN2 */
    AD1CHS123bits.CH123SA = 1;  /* CHx+: AN3, AN0, AN6 */
#elif defined(__EP200__)    
    AD1CSSHbits.CSS18 = 1;      /* AN18 */
    AD1CHS123bits.CH123SA1 = 1; /* CHx+: AN3, AN0, AN6 */
#endif   
    AD1CHS123bits.CH123NA = 0b00;   /* CHx-: Vrefl */    
    
    AD1CON1bits.ADON = 1;       /* enable ADC */       
    

    /*------------------------------------------------------------------------*/
    /** - Configuration of Timer 1 unit for computation time measurement */
    /*------------------------------------------------------------------------*/
    T1CONbits.TCS = 0;      /* internal clock (fCY) */
    T1CONbits.TGATE = 0;    /* timer in counter mode */
    T1CONbits.TCKPS = 0b00; /* 1:1 prescaler */
    TMR1 = 0x0000;          /* clear timer 1 value */
    T1CONbits.TON = 1;      /* start timer 1 */
    
    /*------------------------------------------------------------------------*/
    /** - Configuration of Timer 1 unit for computation time measurement 2 */
    /*------------------------------------------------------------------------*/
    T2CONbits.TCS = 0;      /* internal clock (fCY) */
    T2CONbits.TGATE = 0;    /* timer in counter mode */
    T2CONbits.TCKPS = 0b00; /* 1:1 prescaler */
    TMR2 = 0x0000;          /* clear timer 2 value */
    T2CONbits.TON = 1;      /* start timer 2 */
    
    /*------------------------------------------------------------------------*/
    /** - Configuration of Timer 4 unit for slow control task triggering */
    /*------------------------------------------------------------------------*/
    T4CONbits.TCS = 0;      /* internal clock (fCY) */
    T4CONbits.TGATE = 0;    /* timer in counter mode */
    T4CONbits.TCKPS = 0b00; /* 1:1 prescaler */
    TMR4 = 0x0000;          /* clear timer 4 value */
    T4CONbits.TON = 1;      /* start timer 4 */

   
    /*------------------------------------------------------------------------*/
    /** - Configuration of PWM units
     *    - Center-Aligned PWM mode
     *    - Complementary output mode
     *    - Dead-time disabled
     *    . */
    /*------------------------------------------------------------------------*/
    PTCONbits.PTEN = 0;         /* disable PWM module */
    PTCON2bits.PCLKDIV = 0b000; /* input clock prescaler: 1:1 */
    PTPER = PWM_FREQUENCY;      /* primary master time base period (not used for center-aligned pwm)*/
    
    TRGCON1bits.TRGDIV = PERIOD_SELECT-1;/* trigger output frequency NOTE: period select must not be smaller than 1! */
    TRIG1 = 0;                   /* trigger compare value -> center of period */
    
    /* PWM unit 1 */
    PWMCON1bits.TRGIEN = 0;     /* trigger event interrupts are disabled */
    PWMCON1bits.ITB = 1;        /* independent time base mode (needed for center-aligned PWM) */
    PWMCON1bits.MDCS = 0;       /* PDCx provides duty cycle */
    PWMCON1bits.DTC = 0b10;     /* dead-time function is disabled */
#if defined(__EP100__)
    PWMCON1bits.MTBS = 0;       /* PWM generator uses primary master time base for synchronization and as clock source */
#endif
    PWMCON1bits.CAM = 1;        /* enable center-aligned mode */
    PWMCON1bits.IUE = 0;        /* disable immediate update enable */
    
    IOCON1bits.PENH = 0;        /* PWMx module doesn't control PWMxH pin */
    IOCON1bits.PENL = 0;        /* PWMx module doesn't control PWMxL pin */
    IOCON1bits.POLH = 1;        /* PWMxH pin is active-low */
    IOCON1bits.POLL = 1;        /* PWMxL pin is active-low */
    IOCON1bits.PMOD = 0b00;     /* PWMx pin pair is in complementary output mode */   
    PDC1 = PWM_FREQUENCY >> 1;  /* duty cycle set to 50% */
    PHASE1 = PWM_FREQUENCY;     /* phase shift value (equals period in center-aligned mode) */
    IOCON1bits.CLDAT = 0b11;
    IOCON1bits.FLTDAT = 0b11;
    
    //FCLCON1bits.FLTMOD = 0b11;  /* fault mode disabled */   // TODO: remove after debug!
    FCLCON1bits.FLTMOD = 0b00;  /* fault mode enabled */
    FCLCON1bits.FLTPOL = 1;     /* fault polarity is active low */
    
    /* PWM unit 2 */
    PWMCON2bits.TRGIEN = 0;     /* trigger event interrupts are disabled */
    PWMCON2bits.ITB = 1;        /* independent time base mode (needed for center-aligned PWM) */
    PWMCON2bits.MDCS = 0;       /* PDCx provides duty cycle */
    PWMCON2bits.DTC = 0b10;     /* dead-time function is disabled */
#if defined(__EP100__)
    PWMCON2bits.MTBS = 0;       /* PWM generator uses primary master time base for synchronization and as clock source */
#endif
    PWMCON2bits.CAM = 1;        /* enable center-aligned mode */
    PWMCON2bits.IUE = 0;        /* disable immediate update enable */
    
    IOCON2bits.PENH = 0;        /* PWMx module doesn't control PWMxH pin */
    IOCON2bits.PENL = 0;        /* PWMx module doesn't control PWMxL pin */
    IOCON2bits.POLH = 1;        /* PWMxH pin is active-low */
    IOCON2bits.POLL = 1;        /* PWMxL pin is active-low */
    IOCON2bits.PMOD = 0b00;     /* PWMx pin pair is in complementary output mode */
    PDC2 = PWM_FREQUENCY >> 1;  /* duty cycle set to 50% */
    PHASE2 = PWM_FREQUENCY;     /* phase shift value (equals period in center-aligned mode) */
    IOCON2bits.CLDAT = 0b11;
    IOCON2bits.FLTDAT = 0b11;
    
    FCLCON2bits.FLTMOD = 0b00;  /* fault mode enabled */
    FCLCON2bits.FLTPOL = 1;     /* fault polarity is active low */
    
    /* PWM unit 3 */
    PWMCON3bits.TRGIEN = 0;     /* trigger event interrupts are disabled */
    PWMCON3bits.ITB = 1;        /* independent time base mode (needed for center-aligned PWM) */
    PWMCON3bits.MDCS = 0;       /* PDCx provides duty cycle */
    PWMCON3bits.DTC = 0b10;     /* dead-time function is disabled */
#if defined(__EP100__)
    PWMCON3bits.MTBS = 0;       /* PWM generator uses primary master time base for synchronization and as clock source */
#endif
    PWMCON3bits.CAM = 1;        /* enable center-aligned mode */
    PWMCON3bits.IUE = 0;        /* disable immediate update enable */
    
    IOCON3bits.PENH = 0;        /* PWMx module doesn't control PWMxH pin */
    IOCON3bits.PENL = 0;        /* PWMx module doesn't control PWMxL pin */
    IOCON3bits.POLH = 1;        /* PWMxH pin is active-low */
    IOCON3bits.POLL = 1;        /* PWMxL pin is active-low */
    IOCON3bits.PMOD = 0b00;     /* PWMx pin pair is in complementary output mode */
    PDC3 = PWM_FREQUENCY >> 1;  /* duty cycle set to 50% */
    PHASE3 = PWM_FREQUENCY;     /* phase shift value (equals period in center-aligned mode) */
    IOCON3bits.CLDAT = 0b11;
    IOCON3bits.FLTDAT = 0b11;
    
    FCLCON3bits.FLTMOD = 0b00;  /* fault mode enabled */
    FCLCON3bits.FLTPOL = 1;     /* fault polarity is active low */
    
    
    /*------------------------------------------------------------------------*/
    /** - Configuration of UART 2 for communication with motor driver
     *    - 9600 bps
     *    - 8 data bits
     *    - 1 start bit
     *    - 1 stop bit
     *    - half duplex mode
     *    .
     */
    /*------------------------------------------------------------------------*/
    RPINR19bits.U2RXR = (uint8)41;  /* map UART #2 rx to RP41 */
    RPOR3bits.RP41R = (uint8)3;     /* map UART #2 tx to RP41 */
    U2MODEbits.STSEL = (uint8)0;    /* 1 stop bit */
    U2MODEbits.PDSEL = (uint8)0x00; /* 8-bit data, no parity */

    #if defined(__70MHZ__)
        U2BRG = (uint16)455;        /* baudrate = 9600 bps */
    #elif defined(__40MHZ__)
        U2BRG = (uint16)259;        /* baudrate = 9600 bps */
    #else
        #error CLOCK NOT DEFINED
    #endif

    U2STAbits.UTXISEL0 = 1;         /* interrupt when the last character is shifted out of Transmit Shift Register */
    U2STAbits.UTXISEL1 = 0;
   
    U2STA &= 0xFFF1;                /* clear pending errors */
    U2MODEbits.UARTEN = 0x01;       /* enable UART #2 module */
    

    /*------------------------------------------------------------------------*/
    /** - Enabling of interrupts */
    /*------------------------------------------------------------------------*/
    /* set interrupt priority levels (7 = max priority, 0 = min priority) */
    IPC3bits.AD1IP = 6;     /* ADC -> mainTask priority */
    IPC6bits.T4IP = 5;      /* Timer 4 -> slowTask priority */
    IPC7bits.U2RXIP = 3;    /* UART 2 Rx -> serial communication priority */
    IPC7bits.U2TXIP = 4;    /* UART 2 Tx -> serial communication priority */
    IPC16bits.U1EIP = 4;    /* UART 1 error */

    IFS0bits.AD1IF = 0;     /* clear ADC interrupt flag */
    IEC0bits.AD1IE = 1;     /* enable ADC interrupt */

    IFS1bits.T4IF = 0;      /* clear Timer 4 interrupt flag */
    IEC1bits.T4IE = 1;      /* enable Timer 4 interrupt */
    
    IFS1bits.U2RXIF = 0;    /* clear UART 2 Rx interrupt flag bit */
    IEC1bits.U2RXIE = 1;    /* enable UART 2 Rx interrupt */
    
    IFS4bits.U1EIF = 0;     /* clear UART 1 error interrupt flag */    
    IEC4bits.U1EIE = 1;     /* enable UART 1 error interrupt */
    
    PTCONbits.PTEN = 1;     /* enable PWM module */

}

/**
 * @brief Function to activate reverse protection charge pump.
 */
void initReverseProtectionChargePump(void)
{
#if defined(__EP100__)
    RPOR6bits.RP57R = 0b010000;     /* map OC1 to RP57 */
#elif defined(__EP200__)    
    RPOR7bits.RP57R = 0b010000;     /* map OC1 to RP57 */
#endif
    OC1CON1bits.OCTSEL = 0x7;       /* input clock is Fcy */
    OC1CON1bits.OCM = 3;            /* Single Compare mode: Compare event with OCxR, continuously toggles OCx pin */
    OC1R = 100;                     /* 4kHz @ Fcy=40MHz, 400kHz toggle is 200kHz period */
    OC1RS = 100;
    OC1CON2bits.SYNCSEL = 0b11111;  /* OCRS is sync source */
}
