/* $LastChangedRevision: 10 $
 * $LastChangedDate:: 2016-01-13 08:47:25#$
 *
 * Copyright (c) 2013, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
#include "GlobalDefines.h"
#include "InputControl.h"


/* ADC related macros */
#define ADC_CALIBRATE_MAX      0x07FFU  /* number of values for averaging */


/**
 * @brief Routine to read values from ADC.
 */
void readADC(void)
{
	int32 temp;
	static uint32	Sum_Udc_link = 0U;
	static int16	Cnt_Udc_link = 0;

	/*--------------------------------------------------------------------*/
	/** - Phase currents */
	/*--------------------------------------------------------------------*/
    temp = (int32)ADCValue[1] - (int32)ADCOffset[1];
    LIMIT(temp, INT16_MAX);
#if defined(__EP100__)
    I_PHASE_2 = -(int16)temp;
#elif defined(__EP200__)
    I_PHASE_3 = -(int16)temp;
#endif

    temp = (int32)ADCValue[2] - (int32)ADCOffset[2];
    LIMIT(temp, INT16_MAX);
#if defined(__EP100__)
    I_PHASE_3 = -(int16)temp;
#elif defined(__EP200__)
    I_PHASE_2 = -(int16)temp;
#endif

    temp = -(int32)I_PHASE_2 - (int32)I_PHASE_3;
    LIMIT(temp, INT16_MAX);
    I_PHASE_1 = (int16)temp;


	/*------------------------------------------------------------------------*/
	/** - Angular sensor (analog hall sensor) */
	/*------------------------------------------------------------------------*/
	U_HALL_1 = (int16)(ADCValue[3] ^ 0x8000U);  /* sine signal */
	U_HALL_2 = (int16)(ADCValue[4] ^ 0x8000U);  /* cosine signal */

	
	/*------------------------------------------------------------------------*/
	/** - DC-link voltage */
	/*------------------------------------------------------------------------*/
	Sum_Udc_link += (ADCValue[5] >> 1);

	/* Computation of mean values */
	Cnt_Udc_link++;
	if (Cnt_Udc_link >= 16) /* averaging over  2^4 values */
	{
		/* DC-link voltage */
		U_DCLINK = Sum_Udc_link >> 4;
		Sum_Udc_link = 0U;

		Cnt_Udc_link = 0;
	}

}

/**
 * @brief Routine to calibrate ADC / get current offsets.
 */
void calibrateADC(void)
{
	static uint32 SumI[3];

  	if (CalibrationCounter == 0U)
  	{
		SumI[0] = 0U;
		SumI[1] = 0U;
		SumI[2] = 0U;
  	}
  
	/** - Get ADC results and add to sum */
    #if defined(__EP100__)
        SumI[1] += (uint16)ADCValue[1];	/* Phase current 2 */
        SumI[2] += (uint16)ADCValue[2];	/* Phase current 3 */
    #elif defined(__EP200__)
        SumI[0] += (uint16)ADCValue[0];	/* Phase current 1 */
    	SumI[1] += (uint16)ADCValue[1];	/* Phase current 2 */
    	SumI[2] += (uint16)ADCValue[2];	/* Phase current 3 */
    #else
        #error MOTOR TYPE NOT DEFINED
    #endif
  
  	/** - Check for end of calibration and calculate mean value */
  	if(CalibrationCounter == ADC_CALIBRATE_MAX)
  	{	/*						Mean			    + 		Rounding */
    	ADCOffset[0] = (uint16)(SumI[0] >> 11) + (uint16)((SumI[0] & ADC_CALIBRATE_MAX) >> (11-1));
    	ADCOffset[1] = (uint16)(SumI[1] >> 11) + (uint16)((SumI[1] & ADC_CALIBRATE_MAX) >> (11-1));
    	ADCOffset[2] = (uint16)(SumI[2] >> 11) + (uint16)((SumI[2] & ADC_CALIBRATE_MAX) >> (11-1));

		CalibrationCounter = (uint16)INT16_MAX;
  	}
  	else{
  		CalibrationCounter++;
  	}

}


/**
 * @brief Routine to read digital input pins.
 */
void readDigitalIn(void)
{    
    /*------------------------------------------------------------------------*/
	/** - Read peripheral signals
     *    - Digital_IN
     *    .
     */
	/*------------------------------------------------------------------------*/
	DIGITAL_IN = __builtin_mulss(READ_DIGITAL_IN, INT16_MAX);
    /*
    if (READ_DIGITAL_IN != 0){
        DIGITAL_IN = INT16_MAX;
    }
    else
    {
        DIGITAL_IN = 0;
    }
    */
    /* NOTE: multiplication method saves about 70ns over method with if-statements! */
    
	/*------------------------------------------------------------------------*/
	/** - Read driver signals
     *    - ILIMIT_OUT
     *    - ZC_OUT                                                
     *    .
     */
	/*------------------------------------------------------------------------*/
    ILIMIT_OUT = __builtin_mulss(READ_ILIMIT_OUT, INT16_MAX);
 	ZC_OUT     = __builtin_mulss(READ_ZC_OUT, INT16_MAX);

}

