/* $LastChangedRevision: 6 $
 * $LastChangedDate:: 2015-10-28 15:17:09#$
 *
 * Copyright (c) 2013, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
#include "GlobalDefines.h"
#include "OutputControl.h"


/*----------------------------------------------------------------------------*/
/* Macros                                                                     */
/*----------------------------------------------------------------------------*/
#if defined(__USE_V0_SVM__)
	/** Null = V0 SVM
	 * -------------
	 * compare register setting: ( 0..1 -> 0..PWM_max )
	 *   reg = PWM - x*PWM_max
	 */
	#define SET_PWM_CMPR(OUT, COMPARE_REG)	do{\
												if((OUT) < 0){\
													COMPARE_REG = PWM_FREQUENCY;\
												}else{\
													COMPARE_REG = PWM_FREQUENCY - (__builtin_mulss((OUT), PWM_MAX_ON)) >> 15;\
												}\
											}while(0)
#else
	/** Alternate reversing SVM
	 * -----------------------
	 * compare register setting: ( -1..1 -> (PWM-PWM_max)/2..(PWM+PWM_max)/2 )
	 *   reg = PWM/2 - x*PWM_max/2
	 */
	#define SET_PWM_CMPR(OUT, COMPARE_REG)	((COMPARE_REG) = (PWM_FREQUENCY >> 1) - (uint16)(__builtin_mulss((OUT), PWM_MAX_ON) >> 16))
#endif


/**
 * @brief Routine to set PWM duty cycle.
 */
void setPWM(void)
{

	if (FIState == RUN_STATE)
	{
		/* module has powered up */
		SET_PWM_CMPR(PWM_PHASE_1, PDC1);
		SET_PWM_CMPR(PWM_PHASE_2, PDC2);
		SET_PWM_CMPR(PWM_PHASE_3, PDC3);
	}
	else
	{
		/* module isn't powered up; set PWM value to 50% duty cycle */
		PDC1 = PWM_FREQUENCY >> 1;
		PDC2 = PWM_FREQUENCY >> 1;
		PDC3 = PWM_FREQUENCY >> 1;
	}
}
