/** @file
 * @brief Serial interface setup for Microchip dsPIC33E family.
 */
/* $LastChangedRevision: 10 $
 * $LastChangedDate:: 2016-01-13 08:47:25 +0100#$
 *
 * Copyright (c) 2015 Linz Center of Mechatronics GmbH
 * All rights reserved.
 */
#ifndef __SERIAL33E_H__
#define __SERIAL33E_H__

#include <p33Exxxx.h>
#include "CommonFcts.h"

/** Baudrate for serial interface */
typedef enum {
    BAUDRATE_19200, /**<  19.2kB/s */
    BAUDRATE_38400, /**<  38.4kB/s */
    BAUDRATE_115200 /**< 115.2kB/s */
} tSerialBaudrate;

/** Number of data bits for serial interface */
typedef enum {
    DATABITS_8,     /**< 8 data bits */
    DATABITS_9      /**< 9 data bits */
} tSerialDataBits;

/** Number of stop bits for serial interface */
typedef enum {
    STOPBITS_1,     /**< 1 stop bit */
    STOPBITS_2      /**< 2 stop bits */
} tSerialStopBits;

/** Type of parity mode */
typedef enum {
    PARITY_OFF,     /**< no parity */
    PARITY_ODD,     /**< odd parity */
    PARITY_EVEN     /**< even parity */
} tSerialParity;

/** Mapping of receive pin */
typedef enum {
    RXPIN_RPI53,    /**< remappable input pin 53 */
    RXPIN_RP41      /**< remappable pin 41 */
} tSerialRxPin;

/** Mapping of transmit pin */
typedef enum {
    TXPIN_RP20,     /**< remappable pin 20 */
    TXPIN_RP40,     /**< remappable pin 40 */
    TXPIN_RP97      /**< remappable pin 97 */
} tSerialTxPin;

/** Serial interface structure */
typedef struct {
	void (*send)(tInterface*, uint8);               /**< Function link to send character */
	uint8 (*receive)(tInterface*);                  /**< Function link to receive character */
	uint8 (*isReceiveDataAvailable)(tInterface*);   /**< Function link to check if receive data is available */
	uint8 (*isSendReady)(tInterface*);              /**< Function link to check if data can be sent */
	uint8 (*getTxFifoFree)(tInterface*);            /**< Function link to check if transmit FIFO is ready to be filled */
	void (*flush)(tInterface*);                     /**< Function link to flush data */
    volatile UART* module;                          /**< Link to the hardware module */
} tSerial;

/* public prototypes */
void initSerialA(tSerial* serial33E, tSerialBaudrate baudrate, tSerialDataBits databits, \
        tSerialStopBits stopbits, tSerialParity parity, tSerialRxPin rxPin, tSerialTxPin txPin);
void linkSerial(tProtocol* lnkProtocol, tSerial* serial33E);

#endif
