/*
 * Copyright (c) {YEAR}, {ORGANIZATION_NAME}, {ORGANIZATION_WEB}
 * All rights reserved.
 */
/*
 * This file is licensed according to the BSD 3-clause license as follows:
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of the "{ORGANIZATION_NAME}" nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL "{ORGANIZATION_NAME}" BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * This file is part of X2C. http://www.mechatronic-simulation.org/
 * $LastChangedRevision$
 */
/* USERCODE-BEGIN:Description                                                                                         */
/* Description: */
/* USERCODE-END:Description                                                                                           */
#include "park_inv_INT16_ASM.h"

/* all used update functions to ram for c2000                                                                         */
#if defined(__ALL_UPDATE_FUNC_2_RAM_C2000__)
     #pragma CODE_SECTION(park_inv_INT16_ASM_Update, "ramfuncs")
#endif

/* USERCODE-BEGIN:PreProcessor                                                                                        */
#include "motor_control.h"
#include "Sin_Data.h"


#define D 			*pTpark_inv_INT16_ASM->d
#define Q 			*pTpark_inv_INT16_ASM->q
#define THETA		*pTpark_inv_INT16_ASM->theta

#define ALPHA		pTpark_inv_INT16_ASM->alpha
#define BETA		pTpark_inv_INT16_ASM->beta

#define HALFPIINT16 0x4000	
/* USERCODE-END:PreProcessor                                                                                          */

/**********************************************************************************************************************/
/** Update                                                                                                           **/
/**********************************************************************************************************************/
void park_inv_INT16_ASM_Update(PARK_INV_INT16_ASM *pTpark_inv_INT16_ASM)
{
/* USERCODE-BEGIN:UpdateFnc                                                                                           */
#ifdef __XC16__
    MC_DQ_T mcVDQ;    
    MC_SINCOS_T mcSinCos;    
    MC_ALPHABETA_T mcIAlphaBeta;   
    
    mcVDQ.d = D;
    mcVDQ.q = Q;
    
    MC_CalculateSineCosine_Assembly_Ram(THETA, &mcSinCos);
    MC_TransformParkInverse_Assembly(&mcVDQ, &mcSinCos, &mcIAlphaBeta);
    
    ALPHA = mcIAlphaBeta.alpha;
    BETA = mcIAlphaBeta.beta;
#else 
	int16 sin_theta; 
	int16 cos_theta;
	
	int32 temp1;
	int32 temp2;


	LOOKUP16(Sin_Table16, HALFPIINT16 - THETA, cos_theta);
	LOOKUP16(Sin_Table16, THETA, sin_theta);
	
	temp1 = (int16)(((int32)D * (int32)cos_theta)>>15) - (int16)(((int32)Q * (int32)sin_theta)>>15);
	temp2 = (int16)(((int32)D * (int32)sin_theta)>>15) + (int16)(((int32)Q * (int32)cos_theta)>>15);
	
	/* output limitation */
	if (temp1 > INT16_MAX)
	{
		temp1 = INT16_MAX;
	}
	else
	{
		if (temp1 < -INT16_MAX)
		{
			temp1 = -INT16_MAX;
		}
	}
	
	if (temp2 > INT16_MAX)
	{
		temp2 = INT16_MAX;
	}
	else
	{
		if (temp2 < -INT16_MAX)
		{
			temp2 = -INT16_MAX;
		}
	}
	
	ALPHA = (int16)(temp1);
	BETA  = (int16)(temp2);
#endif
/* USERCODE-END:UpdateFnc                                                                                             */

}

/**********************************************************************************************************************/
/** Initialization                                                                                                   **/
/**********************************************************************************************************************/
void park_inv_INT16_ASM_Init(PARK_INV_INT16_ASM *pTpark_inv_INT16_ASM)
{
     pTpark_inv_INT16_ASM->ID = PARK_INV_INT16_ASM_ID;
     pTpark_inv_INT16_ASM->alpha = 0;
     pTpark_inv_INT16_ASM->beta = 0;
/* USERCODE-BEGIN:InitFnc                                                                                             */
/* USERCODE-END:InitFnc                                                                                               */
}

/**********************************************************************************************************************/
/** No Load function                                                                                                 **/
/**********************************************************************************************************************/

/**********************************************************************************************************************/
/** No Save function                                                                                                 **/
/**********************************************************************************************************************/

/**********************************************************************************************************************/
/** Get block element address                                                                                        **/
/**********************************************************************************************************************/
#if !defined(PARK_INV_INT16_ASM_ISLINKED)
void* park_inv_INT16_ASM_GetAddress(const PARK_INV_INT16_ASM* block, uint16 elementId)
{
    void* addr;
    switch (elementId)
    {
        case 1:
            addr = (void*)block->d;
            break;
        case 2:
            addr = (void*)block->q;
            break;
        case 3:
            addr = (void*)block->theta;
            break;
        case 4:
            addr = (void*)&block->alpha;
            break;
        case 5:
            addr = (void*)&block->beta;
            break;
        default:
            addr = (void*)0;
            break;
    }
    return (addr);
}
#endif
